from unittest import skip

from django.test import TestCase

from browser_logging.models import BrowserLogData


class BrowserLoggingTests(TestCase):
    def test_simple_request(self):
        self.assertEqual(BrowserLogData.objects.count(), 0)

        self.client.get("/test")
        self.client.get("/test")

        self.assertEqual(BrowserLogData.objects.count(), 1)

    def test_samsung_galaxy_s9(self):
        user_agent_string = (
            "Mozilla/5.0 (Linux; Android 8.0.0; SM-G960F Build/R16NW) "
            "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.84 "
            "Mobile Safari/537.36"
        )

        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)
        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)

        bld = BrowserLogData.objects.get()
        self.assertFalse(bld.is_tablet)
        self.assertTrue(bld.is_mobile)
        self.assertEqual(bld.browser_family, "Chrome Mobile")
        self.assertFalse(bld.is_bot)
        self.assertFalse(bld.is_pc)
        self.assertEqual(bld.os_family, "Android")
        self.assertEqual(bld.os_version, "8.0.0")

    def test_google_pixel_6(self):
        user_agent_string = (
            "Mozilla/5.0 (Linux; Android 12; Pixel 6 Build/SD1A.210817.023; wv) "
            "AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/94.0.4606.71 "
            "Mobile Safari/537.36"
        )

        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)
        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)

        bld = BrowserLogData.objects.get()
        self.assertFalse(bld.is_tablet)
        self.assertTrue(bld.is_mobile)
        self.assertEqual(bld.browser_family, "Chrome Mobile WebView")
        self.assertFalse(bld.is_bot)
        self.assertFalse(bld.is_pc)
        self.assertEqual(bld.os_family, "Android")
        self.assertEqual(bld.os_version, "12")

    def test_apple_iphone_with_ios_15_4(self):
        user_agent_string = (
            "Mozilla/5.0 (iPhone14,6; U; CPU iPhone OS 15_4 like Mac OS X) "
            "AppleWebKit/602.1.50 (KHTML, like Gecko) Version/10.0 Mobile/19E241 "
            "Safari/602.1"
        )

        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)
        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)

        bld = BrowserLogData.objects.get()
        self.assertFalse(bld.is_tablet)
        self.assertTrue(bld.is_mobile)
        self.assertEqual(bld.browser_family, "Mobile Safari")
        self.assertFalse(bld.is_bot)
        self.assertFalse(bld.is_pc)
        self.assertEqual(bld.os_family, "iOS")
        self.assertEqual(bld.os_version, "15.4")

    def test_amazon_kindle_fire_hdx_7(self):
        user_agent_string = (
            "Mozilla/5.0 (Linux; Android 4.4.3; KFTHWI Build/KTU84M) AppleWebKit/537.36"
            " (KHTML, like Gecko) Silk/47.1.79 like Chrome/47.0.2526.80 Safari/537.36"
        )

        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)
        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)

        bld = BrowserLogData.objects.get()
        self.assertTrue(bld.is_tablet)
        self.assertFalse(bld.is_mobile)
        self.assertEqual(bld.browser_family, "Amazon Silk")
        self.assertFalse(bld.is_bot)
        self.assertFalse(bld.is_pc)
        self.assertEqual(bld.os_family, "Android")
        self.assertEqual(bld.os_version, "4.4.3")

    @skip
    def test_samsung_galaxy_tab_s8_ultra(self):
        user_agent_string = (
            "Mozilla/5.0 (Linux; Android 12; SM-X906C Build/QP1A.190711.020; wv) "
            "AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/80.0.3987.119 "
            "Mobile Safari/537.36"
        )

        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)
        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)

        bld = BrowserLogData.objects.get()
        self.assertTrue(bld.is_tablet)
        self.assertFalse(bld.is_mobile)
        self.assertEqual(bld.browser_family, "Chrome Mobile")
        self.assertFalse(bld.is_bot)
        self.assertFalse(bld.is_pc)
        self.assertEqual(bld.os_family, "Android")
        self.assertEqual(bld.os_version, "12")

    def test_apple_ipad_with_ios_9(self):
        user_agent_string = (
            "Mozilla/5.0 (iPad; CPU OS 9_3_5 like Mac OS X) AppleWebKit/601.1.46 "
            "(KHTML, like Gecko) Version/9.0 Mobile/13G36 Safari/601.1"
        )

        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)
        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)

        bld = BrowserLogData.objects.get()
        self.assertTrue(bld.is_tablet)
        self.assertFalse(bld.is_mobile)
        self.assertEqual(bld.browser_family, "Mobile Safari")
        self.assertFalse(bld.is_bot)
        self.assertFalse(bld.is_pc)
        self.assertEqual(bld.os_family, "iOS")
        self.assertEqual(bld.os_version, "9.3.5")

    def test_google_bot(self):
        user_agent_string = (
            "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)"
        )

        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)
        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)

        bld = BrowserLogData.objects.get()
        self.assertFalse(bld.is_tablet)
        self.assertFalse(bld.is_mobile)
        self.assertEqual(bld.browser_family, "Googlebot")
        self.assertTrue(bld.is_bot)
        self.assertFalse(bld.is_pc)
        self.assertEqual(bld.os_family, "Other")
        self.assertEqual(bld.os_version, "")

    def test_google_chrome_windows_10(self):
        user_agent_string = (
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 "
            "(KHTML, like Gecko) Chrome/106.0.0.0 Safari/537.36"
        )

        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)
        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)

        bld = BrowserLogData.objects.get()
        self.assertFalse(bld.is_tablet)
        self.assertFalse(bld.is_mobile)
        self.assertEqual(bld.browser_family, "Chrome")
        self.assertFalse(bld.is_bot)
        self.assertTrue(bld.is_pc)
        self.assertEqual(bld.os_family, "Windows")
        self.assertEqual(bld.os_version, "10")

    def test_firefox_windows_10(self):
        user_agent_string = (
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:105.0) Gecko/20100101 "
            "Firefox/105.0"
        )

        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)
        self.client.get("/test", HTTP_USER_AGENT=user_agent_string)

        bld = BrowserLogData.objects.get()
        self.assertFalse(bld.is_tablet)
        self.assertFalse(bld.is_mobile)
        self.assertEqual(bld.browser_family, "Firefox")
        self.assertFalse(bld.is_bot)
        self.assertTrue(bld.is_pc)
        self.assertEqual(bld.os_family, "Windows")
        self.assertEqual(bld.os_version, "10")
